@@ -49,6 +49,7 @@ namespace po = boost::program_options;
 #ifdef _WIN32
 #  include <malloc.h>
 #endif
+#include <unistd.h>
 #ifndef NDEBUG
 const bool debug=true;
 #else
@@ -80,11 +81,25 @@ boost::asio::io_service io_service;
 using boost::asio::ip::udp;
 boost::scoped_ptr<udp::socket> udp_socket;
 boost::scoped_ptr<udp::endpoint> udp_endpoint;
+std::string hostname = "";
 
 bool is_verbose()
 {
   return debug || verbose || ponder_enabled || error_log_file != "";
 }
+void send_udp_log(std::string line) 
+{
+  if (! udp_socket)
+    return;
+  assert(line.find('\n') == line.npos);
+  boost::system::error_code ignored_error;
+  line += "\n";
+  if (hostname != "")
+    line = hostname + " " + line;
+  udp_socket->send_to(boost::asio::buffer(line.c_str(), line.length()),
+		      *udp_endpoint, 0,
+		      ignored_error);
+}
 
 volatile int go_count = 0;
 volatile int bestmove_count = 0;
@@ -204,6 +219,7 @@ public:
     }
     if (is_verbose())
       std::cerr << "sent: " << result << "\n";
+    send_udp_log(result);
     suspend_reading = false;
     boost::mutex::scoped_lock lk(go_stop_mutex);
     ++bestmove_count;
@@ -387,8 +403,12 @@ void GoMateThread::operator()()
   NumEffectState state(input_state->currentState());
 #if (! defined ALLOW_KING_ABSENCE)
   if (state.kingSquare(state.turn()).isPieceStand()) {
-    boost::mutex::scoped_lock lk(OslConfig::lock_io);
-    os << "checkmate notimplemented\n";
+    const char *msg = "checkmate notimplemented";
+    {
+      boost::mutex::scoped_lock lk(OslConfig::lock_io);
+      os << msg << "\n";
+    }
+    send_udp_log(msg);
     return;
   }
 #endif  
@@ -455,21 +475,31 @@ void GoMateThread::operator()()
   if (is_verbose())
     std::cerr << "elapsed " << start.elapsedSeconds() << "\n";
   if (! result.isFinal()) {
-    boost::mutex::scoped_lock lk(OslConfig::lock_io);
-    os << "checkmate timeout\n" << std::flush;
+    const char *msg = "checkmate timeout";
+    {
+      boost::mutex::scoped_lock lk(OslConfig::lock_io);
+      os << msg << "\n" << std::flush;
+    }
+    send_udp_log(msg);
     return;
   }
   if (! result.isCheckmateSuccess()) {
-    boost::mutex::scoped_lock lk(OslConfig::lock_io);
-    os << "checkmate nomate\n" << std::flush;
-;
+    const char *msg = "checkmate nomate";
+    {
+      boost::mutex::scoped_lock lk(OslConfig::lock_io);
+      os << msg << "\n" << std::flush;
+    }
+    send_udp_log(msg);
     return;
   }
-  boost::mutex::scoped_lock lk(OslConfig::lock_io);
-  os << "checkmate";
+  std::string msg = "checkmate";
   for (size_t i=0; i<pv.size(); ++i)
-    os << " " << record::usi::show(pv[i]);
-  os << "\n" << std::flush;
+    msg += " " + record::usi::show(pv[i]);
+  {
+    boost::mutex::scoped_lock lk(OslConfig::lock_io);
+    os << msg << "\n" << std::flush;
+  }
+  send_udp_log(msg);
 }
 
 void do_checkmate(double seconds, std::ostream& os)
@@ -495,19 +525,26 @@ void play_by_book(int depth_limit, std::ostream& os)
       const Move best_move = tracer.selectMove();
       if (best_move.isNormal()
 	  && (! state.isIllegal(best_move))) {
-	boost::mutex::scoped_lock lk(OslConfig::lock_io);
 	std::string result = "bestmove " + record::usi::show(best_move);
-	os << result << "\n" << std::flush;
+	{
+	  boost::mutex::scoped_lock lk(OslConfig::lock_io);
+	  os << result << "\n" << std::flush;
+	}
 	if (is_verbose())
 	  std::cerr << "sent: " << result << "\n";
+	send_udp_log(result);
 	return;
       }
     }
   }
-  boost::mutex::scoped_lock lk(OslConfig::lock_io);
-  os << "bestmove pass\n" << std::flush;
+  const char *msg = "bestmove pass";
+  {
+    boost::mutex::scoped_lock lk(OslConfig::lock_io);
+    os << msg << "\n" << std::flush;
+  }
   if (is_verbose())
-    std::cerr << "sent: " << "bestmove pass\n";
+    std::cerr << "sent: " << msg << "\n";
+  send_udp_log(msg);
 }
 
 void play_declare_win(std::ostream& os)
@@ -515,11 +552,14 @@ void play_declare_win(std::ostream& os)
   const bool can_declare_win
     = EnterKing::canDeclareWin(input_state->currentState());
   const std::string msg = std::string("bestmove ")
-    + (can_declare_win ? "win" : "pass") + "\n" ;
-  boost::mutex::scoped_lock lk(OslConfig::lock_io);
-  os << msg << std::flush;
+    + (can_declare_win ? "win" : "pass");
+  {
+    boost::mutex::scoped_lock lk(OslConfig::lock_io);
+    os << msg << "\n" << std::flush;
+  }
   if (is_verbose())
-    std::cerr << "sent: " << msg;
+    std::cerr << "sent: " << msg << "\n";
+  send_udp_log(msg);
 }
 
 std::string make_filename()
@@ -777,6 +817,8 @@ void play(std::istream& is, std::ostream& os)
 #ifdef OSL_SMP
   name += "(smp) ";
 #endif
+  if (hostname != "")
+    name += hostname + " ";
   os << name << gpsshogi::gpsshogi_revision << "\n";
   os << "id author teamgps\n";
   os << "option name LimitDepth type spin default " << limit_depth << " min 4 max 10\n";
@@ -824,15 +866,7 @@ void play(std::istream& is, std::ostream& os)
     }
     if (input_log_stream)
       *input_log_stream << line << "\n" << std::flush;
-    if (udp_socket) {
-      boost::system::error_code ignored_error;
-      line += "\n";
-      udp_socket->send_to(boost::asio::buffer(line.c_str(), line.length()),
-                          *udp_endpoint, 0,
-			  ignored_error);
-      line.resize(line.size()-1);
-    }  
-    
+    send_udp_log(line);    
     if (line == "isready") {
       static bool initalized = false;
       if (! initalized) {
@@ -844,9 +878,11 @@ void play(std::istream& is, std::ostream& os)
 	lightweight_commands.hasImmediateResponse("genmove_probability", response); // setup
 	initalized = true;
       }
-      os << "readyok\n";
+      const char *msg = "readyok";
+      os << msg << "\n";
       if (debug)
-	std::cerr << "readyok\n";
+	std::cerr << msg << "\n";
+      send_udp_log(msg);
       continue;
     }
     if (line.find("setoption") == 0) {
@@ -874,8 +910,11 @@ void play(std::istream& is, std::ostream& os)
     }
 
     if (lightweight_commands.hasImmediateResponse(line, response)) {
-      boost::mutex::scoped_lock lk(OslConfig::lock_io);
-      os << response << "\n" << std::flush;
+      {
+	boost::mutex::scoped_lock lk(OslConfig::lock_io);
+	os << response << "\n" << std::flush;
+      }
+      send_udp_log(response);
       continue;
     }
 
@@ -1177,6 +1216,12 @@ int main(int argc, char **argv)
       << "Copyright (C) 2003-2011 Team GPS.\n";
     return 0;
   }
+  char name[128] = {0};
+  if (gethostname(name, 100) == 0) {
+    hostname = name;
+    if (hostname.find(".") != hostname.npos)
+      hostname.resize(hostname.find("."));
+  }
   osl::OslConfig::setMemoryUsePercent(memory_use_percent);
 #ifndef MINIMAL
   if (eval_random)
@@ -1299,7 +1344,7 @@ game_playing::ComputerPlayer *makePlayer(Player turn, const SimpleState& initial
   const bool extended_usi = OslConfig::usiMode() == OslConfig::ExtendedUSI;
   boost::shared_ptr<UsiMonitor> monitor(new UsiMonitor(extended_usi, os));
   if (udp_socket)
-    monitor->setUdpLogging(udp_socket.get(), udp_endpoint.get());
+    monitor->setUdpLogging(hostname, udp_socket.get(), udp_endpoint.get());
 
   search_player->addMonitor(monitor);
   bool use_book = book_depth > 0;
