@@ -1,28 +1,36 @@
 #!/usr/bin/perl -w
 use strict;
 use Getopt::Std;
+use File::Path;
+use Time::Piece;
 
 # sync.pl -l localdir -h host -r remotedir
 # -- localdir/in -> remotedir@host/in
 # -- remotedir@host/out -> remotedir@host/out
 
 my %options;
-getopts("l:h:r:",\%options);
+getopts("l:h:r:N",\%options);
 
-my ($localdir, $host, $remotedir, $lpath, $rpath)
+my ($localdir, $host, $remotedir, $lpath, $rpath, $newgame)
   = ($options{l} || "local-csa",
-     $options{h} || "macpro",
-     $options{r} || "/home/wcsc21/remote-csa",
+     $options{h} || "komaba",
+     $options{r} || "/home/wcsc22/remote-csa",
      $options{L} || "..",
-     $options{R} || "/home/wcsc21",
+     $options{R} || "/home/wcsc22",
+     $options{N},
     );
 
-system "mkdir", "-p", "$localdir/in";
-system "mkdir", "-p", "$localdir/out";
-system "ssh $host mkdir -p $remotedir/in";
-system "ssh $host mkdir -p $remotedir/out";
-system "ssh $host chmod 777 $remotedir/in";
-system "ssh $host chmod 777 $remotedir/out";
+my $now = Time::Piece::localtime;
+if ($newgame) {
+    rename $localdir, $localdir."-".$now->datetime();
+    system "ssh", $host, "mv $remotedir $remotedir-".$now->datetime();
+}
+mkpath("$localdir/in", 1, 0755);
+mkpath("$localdir/out", 1, 0755);
+
+my $initialize = "mkdir -p $remotedir/in; mkdir -p $remotedir/out; "
+    ."chmod 755 $remotedir/in; chmod 755 $remotedir/out";
+system "ssh", $host, $initialize;
 
 {
   my $pid=fork;
@@ -43,6 +51,8 @@ system "ssh $host chmod 777 $remotedir/out";
     exit(0);
   }
 }
+sleep(3);
+print STDERR "hopefully ready\n";
 
 wait;
 wait;
