@@ -9,7 +9,7 @@
 #include <sstream>
 
 gpsshogi::gui::EditBoard2::EditBoard2(QWidget *parent)
-  : AbstractBoard(parent), modifiers(Qt::NoModifier)
+  : AbstractBoard(parent), modifiers(Qt::NoModifier), turn(osl::BLACK)
 {
   initPieces<osl::PAWN>();
   initPieces<osl::LANCE>();
@@ -94,21 +94,25 @@ void gpsshogi::gui::EditBoard2::mousePressEvent(QMouseEvent *e)
 {
   if (e->button() != Qt::LeftButton || modifiers != Qt::NoModifier)
     return;
-  osl::Piece piece = getPiece(e->x(), e->y());
+  const int x = e->x() / scale_mult;
+  const int y = e->y() / scale_mult;
+  osl::Piece piece = getPiece(x, y);
   selectedPiece = piece;
   if (!selectedPiece.isEmpty())
   {
-    QCursor cursor(pieceToImage(selectedPiece));
-    setCursor(cursor);
+    mousePoint = e->pos();
     update();
   }
 }
 
 void gpsshogi::gui::EditBoard2::mouseReleaseEvent(QMouseEvent *e)
 {
+  const int x = e->x() / scale_mult;
+  const int y = e->y() / scale_mult;
+
   if (!selectedPiece.isEmpty())
   {
-    osl::Square pos = getSquare(e->x(), e->y());
+    osl::Square pos = getSquare(x, y);
     if (pos.isValid() && pos.isOnBoard() && pieceAt(pos).isEmpty())
     {
       // 盤上
@@ -124,14 +128,14 @@ void gpsshogi::gui::EditBoard2::mouseReleaseEvent(QMouseEvent *e)
       board[pos.index()] = piece;
     }
     // 駒台
-    else if ((e->x() >= BLACK_STAND_X || e->x() <= STAND_SIZE) &&
+    else if ((x >= BLACK_STAND_X || x <= STAND_SIZE) &&
 	     selectedPiece.ptype() != osl::KING)
     {
       if (selectedPiece.square().isValid() && selectedPiece.isOnBoard())
       {
 	board[selectedPiece.square().index()] = osl::Piece::EMPTY();
       }
-      osl::Piece piece(e->x() >= BLACK_STAND_X ? osl::BLACK : osl::WHITE,
+      osl::Piece piece(x >= BLACK_STAND_X ? osl::BLACK : osl::WHITE,
 		       osl::unpromote(selectedPiece.ptype()),
 		       selectedPiece.number(), osl::Square::STAND());
       pieces[selectedPiece.number()] = piece;
@@ -141,7 +145,7 @@ void gpsshogi::gui::EditBoard2::mouseReleaseEvent(QMouseEvent *e)
   else if (e->button() == Qt::MidButton ||
 	   (e->button() == Qt::LeftButton && modifiers & Qt::SHIFT))
   {
-    osl::Square pos = getSquare(e->x(), e->y());
+    osl::Square pos = getSquare(x, y);
     if (pos.isValid() && pos.isOnBoard() && !pieceAt(pos).isEmpty())
     {
       const osl::Piece p = pieceAt(pos);
@@ -158,7 +162,7 @@ void gpsshogi::gui::EditBoard2::mouseReleaseEvent(QMouseEvent *e)
   // 反転
   else if (e->button() == Qt::RightButton)
   {
-    osl::Square pos = getSquare(e->x(), e->y());
+    osl::Square pos = getSquare(x, y);
     if (pos.isValid() && pos.isOnBoard() && !pieceAt(pos).isEmpty())
     {
       const osl::Piece p = pieceAt(pos);
@@ -215,8 +219,7 @@ osl::state::SimpleState gpsshogi::gui::EditBoard2::getState()
   {
     sstream << "P-" << white_stand.str() << std::endl;
   }
-  // TODO: white turn
-  sstream << "+" << std::endl;
+  sstream << (turn == osl::BLACK ? "+" : "-") << std::endl;
   const std::string &board = sstream.str();
   osl::record::csa::CsaString csa(board.c_str());
   try
@@ -295,3 +298,8 @@ void gpsshogi::gui::EditBoard2::reserveToStand(osl::Player player)
   }
   update();
 }
+
+void gpsshogi::gui::EditBoard2::setTurn(osl::Player player)
+{
+  turn = player;
+}
