@@ -571,9 +571,10 @@ boost::tuple<double,double,double> run_cross_validation(int cross_start, int cro
     };
   boost::ptr_vector<boost::thread> threads;
       
+  std::vector<std::string> csa_files;
   KisenAnalyzer::RecordConfig cross_configs[MaxThreads];
   KisenAnalyzer::distributeJob(num_cpus, &cross_configs[0], cross_start, cross_num,
-			       kisen_filenames, min_rating);
+			       kisen_filenames, min_rating, csa_files);
   for (size_t j=0; j<num_cpus; ++j) {
     cross_configs[j].allow_skip_in_cross_validation = allow_skip;
     threads.push_back(new boost::thread(Validator(cross_configs[j], config, results+j)));
@@ -597,8 +598,9 @@ boost::tuple<double,double,double> run_cross_validation(int cross_start, int cro
 void analyze_gradient(const std::string& output_prefix,
 		      size_t kisen_start, size_t num_records)
 {
+  std::vector<std::string> csa_files;
   KisenAnalyzer::distributeJob(num_cpus, &record_configs[0], kisen_start, num_records,
-			       kisen_filenames, min_rating);
+			       kisen_filenames, min_rating, csa_files);
 
   valarray_t w(my_eval->dimension()), gradient(w.size());
   my_eval->saveWeight(&w[0]);
@@ -628,8 +630,9 @@ void analyze_loss(const std::string& output_prefix,
 		  const std::string& gradient_filename,
 		  size_t num_records_loss_analyses)
 {
+  std::vector<std::string> csa_files;
   KisenAnalyzer::distributeJob(num_cpus, &record_configs[0], kisen_start, num_records,
-			       kisen_filenames, min_rating);
+			       kisen_filenames, min_rating, csa_files);
 
   valarray_t w(my_eval->dimension()), gradient(w.size()), stable_w(w.size());
   my_eval->saveWeight(&w[0]);
@@ -746,9 +749,10 @@ void learn_main(const std::string& output_prefix,
 	  max_progress, position_randomness+osl::random(),
 	  min_rating, my_eval.get()
 	};
+      std::vector<std::string> csa_files;
       boost::ptr_vector<boost::thread> threads;
       KisenAnalyzer::distributeJob(num_cpus, &record_configs[0], kisen_start, num_records,
-				   kisen_filenames, min_rating);
+				   kisen_filenames, min_rating, csa_files);
       for (size_t j=0; j<num_cpus; ++j) {
 	threads.push_back(new boost::thread(PVGenerator(pv_base, 
 							record_configs[j], config, results+j)));
