@@ -8,6 +8,7 @@
 #include "pvVector.h"
 #include "pvGenerator.h"
 #include "rotateRecord.h"
+#include "osl/record/csaRecord.h"
 #include "osl/record/kisen.h"
 #include "osl/state/numEffectState.h"
 #include "osl/eval/ml/openMidEndingEval.h"
@@ -28,6 +29,7 @@
 #include <valarray>
 #include <string>
 #include <iostream>
+#include <iomanip>
 #include <ctime>
 #include <unistd.h>
 
@@ -50,6 +52,7 @@ double lambda;
 size_t min_rating = 1500, min_frequency, min_frequency_record;
 double search_window, search_window_for_validation;
 std::vector<std::string> kisen_filenames;
+std::vector<std::string> csa_filenames;
 double scaling_factor, eta_increment, sigmoid_asymmetry;
 size_t num_cpus;
 std::string pv_base;
@@ -72,7 +75,7 @@ int main(int argc, char **argv)
   int cross_start;
   int max_progress;
   bool high_rating_only, compare_pass;
-  int normal_depth=2, quiesce_depth, use_percent, limit_sibling;
+  int normal_depth, quiesce_depth, use_percent, limit_sibling;
 
   options.add_options()
     ("num-records,n",
@@ -95,6 +98,8 @@ int main(int argc, char **argv)
      "start id of kisen records")
     ("kisen-file,k", po::value<std::vector<std::string> >(),
      "filename for records to be analyzed")
+    ("csa-file", po::value<std::vector<std::string> >(&csa_filenames)->multitoken(),
+     "filename for records to be analyzed")
     ("eval,e",
      po::value<std::string>(&eval_type)->default_value(std::string("piece")),
      "evaluation function (piece, rich0, rich1)")
@@ -132,7 +137,7 @@ int main(int argc, char **argv)
      "scale to modify eta")
     ("loss-function,l",
      po::value<std::string>(&loss_function)->default_value(std::string("log")),
-     "loss function (log, hinge, exp, sigmoid)")
+     "loss function (log, hinge, exp, sigmoid, halfsigmoid)")
 #ifndef L1BALL_NO_SORT
     ("smd",
      po::value<bool>(&smd)->default_value(0),
@@ -148,7 +153,7 @@ int main(int argc, char **argv)
      po::value<int>(&limit_sibling)->default_value(0),
      "consider only at most n moves that have higher evaluation values than that of recorded move for each position")
     ("accelerate-minor-features",
-     po::value<bool>(&accelerate_minor_features)->default_value(1),
+     po::value<bool>(&accelerate_minor_features)->default_value(0),
      "try larger step size in update the weights of less frequent features (only effective when smd == 0)")
     ("frequency-min",
      po::value<size_t>(&min_frequency)->default_value(200),
@@ -168,6 +173,9 @@ int main(int argc, char **argv)
     ("watch-threshold",
      po::value<int>(&watch_threshold_seed)->default_value(-10),
      "report details about features whose absolute value is greater than this, or negative value for top-k values")
+    ("depth",
+     po::value<int>(&normal_depth)->default_value(2),
+     "adjust quiescence depth")
     ("quiesce-depth",
      po::value<int>(&quiesce_depth)->default_value(4),
      "adjust quiescence depth")
@@ -321,10 +329,19 @@ void ProcessPVBase::operator()() const
     ++num_position;
     if (record != cur_record) {
       cur_record = record;
-      moves = kisen_file.getMoves(cur_record);
-      finishRecord();
+      if (record >= 0)
+      {
+        moves = kisen_file.getMoves(cur_record);
+        state = NumEffectState(kisen_file.getInitialState());
+      }
+      else
+      {
+        CsaFile csa(record_configs[thread_id].csa_filenames[-record - 1]);
+        state = csa.getInitialState();
+        moves = csa.getRecord().getMoves();
+      }
       cur_position = 0;
-      state = NumEffectState(kisen_file.getInitialState());
+      finishRecord();
     }
     assert(position > cur_position || cur_position == 0);
     while (position > cur_position) {
@@ -349,6 +366,10 @@ void ProcessPVBase::operator()() const
       if (weight_winner && moves.size() < 256 
 	  && (((int)moves.size() % 2) == (cur_position % 2))) // moves[size] corresponds to toryo
 	weight = std::min(0.5, weight);
+      if (record < 0)
+      {
+        weight *= 5;
+      }
       processInstance(turn_coef, selected, sibling, weight, progress);
     }
     endPosition();
@@ -440,6 +461,8 @@ void ProcessPV::endPosition() const
 #ifdef L1BALL_NO_SORT
   if (loss_function == "sigmoid")
     SigmoidLoss::addGradientSep(selected.diffs, count, min_count, agsum, gradient[thread_id]);
+  else if (loss_function == "halfsigmoid")
+    HalfSigmoidLoss::addGradientSep(selected.diffs, count, min_count, agsum, gradient[thread_id]);
   else if (loss_function == "hinge")
     HingeLoss::addGradientSep(selected.diffs, count, min_count, agsum, gradient[thread_id]);
   else
@@ -460,6 +483,10 @@ void ProcessPV::processInstance(double turn_coef, MoveData& selected, MoveData&
     loss = SigmoidLoss::addGradientSep(w, adot, this->selected.diffs, sibling.diffs, turn_coef, count, min_count,
 				       gradient[thread_id], agsum, weight,
 				       margin);
+  else if (loss_function == "halfsigmoid")
+    loss = HalfSigmoidLoss::addGradientSep(w, adot, this->selected.diffs, sibling.diffs, turn_coef, count, min_count,
+				       gradient[thread_id], agsum, weight,
+				       margin);
   else if (loss_function == "hinge")
     loss = HingeLoss::addGradientSep(w, adot, this->selected.diffs, sibling.diffs, turn_coef, count, min_count,
 				       gradient[thread_id], agsum, weight,
@@ -470,6 +497,7 @@ void ProcessPV::processInstance(double turn_coef, MoveData& selected, MoveData&
 				   margin);
   if ((loss_function == "hinge" && loss > 0.0)
       || (loss_function == "sigmoid" && loss > 0.5)
+      || (loss_function == "halfsigmoid" && loss > 0.5)
       || (loss_function == "log" && loss > log(2.0)))
     instance_weight *= decay_sibling_weight;
 #else
@@ -566,13 +594,6 @@ void GradientDescentUtil::iterationHead(int i, const valarray_t& w, double error
   prev_error = error;
   if (i % 10 == 0)
     std::cerr << error;
-  if (i % 50 == 49)
-    std::cerr << std::endl << ' ';
-  if (i % 50 == 49) {
-    for (int i=0; i<std::min(16, (int)w.size()); ++i)
-      std::cerr << "  " << w[i];
-    std::cerr << "\n";
-  }
 }
 
 class GradientDescent : public gpsshogi::L1Ball, GradientDescentUtil
@@ -744,7 +765,8 @@ boost::tuple<double,double,double> run_cross_validation(int cross_start, int cro
       
   KisenAnalyzer::RecordConfig cross_configs[MaxThreads];
   KisenAnalyzer::distributeJob(num_cpus, &cross_configs[0], cross_start, cross_num,
-			       kisen_filenames, min_rating);
+			       kisen_filenames, min_rating,
+                               csa_filenames);
   for (size_t j=0; j<num_cpus; ++j) {
     cross_configs[j].allow_skip_in_cross_validation = allow_skip;
     threads.push_back(new boost::thread(Validator(cross_configs[j], config, results+j)));
@@ -831,10 +853,11 @@ void analyze(const std::string& output_prefix,
       watch_list.clear();
       for (size_t i=my_eval->lambdaStart(); i<my_eval->dimension(); ++i) {
 	if (abs(my_eval->flatValue(i)) > watch_threshold) {
-	  os << "  " << i << " " << my_eval->flatValue(i);
+	  os << " " << std::setw(6) << i << " " << std::setw(3) << my_eval->flatValue(i);
 	  boost::tuple<std::string, int, int> name_index_dim = my_eval->findFeature(i);
-	  os << "  " << name_index_dim.get<0>() << " " << name_index_dim.get<1>()
-	     << " / " << name_index_dim.get<2>() << "  ";
+	  os << " " << std::setw(14) <<name_index_dim.get<0>()
+	     << " "  << std::setw(6) << name_index_dim.get<1>()
+	     << " / " << std::setw(6) << name_index_dim.get<2>() << "  ";
 	  os << my_eval->describe(name_index_dim.get<0>(), name_index_dim.get<1>())
 	     << "\n";
 	  watch_list.push_back(i);
@@ -887,7 +910,7 @@ void analyze(const std::string& output_prefix,
 	};
       boost::ptr_vector<boost::thread> threads;
       KisenAnalyzer::distributeJob(num_cpus, &record_configs[0], kisen_start, num_records,
-				   kisen_filenames, min_rating);
+				   kisen_filenames, min_rating, csa_filenames);
       for (size_t j=0; j<num_cpus; ++j) {
 	threads.push_back(new boost::thread(PVGenerator(pv_base, 
 							record_configs[j], config, results+j)));
@@ -919,8 +942,10 @@ void analyze(const std::string& output_prefix,
       os << " watch list\n";
       for (size_t i=0; i<watch_list.size(); ++i) {
 	const size_t index = watch_list[i];
-	os << "  " << index << " " << my_eval->findFeature(index).get<0>() 
-	   << " count " << count[index] << " record " << count_by_record[0][index] << "\n";
+	os << "  " << index << " "
+	   << std::setw(14) << my_eval->findFeature(index).get<0>() 
+	   << " count " << std::setw(5) << count[index]
+	   << " record " << std::setw(5) << count_by_record[0][index] << "\n";
       }
     }
     if (clear_infrequent) {
