@@ -75,13 +75,13 @@ Eval::maxActive() const
 const std::string gpsshogi::
 Eval::describe(const std::string& feature, size_t local_index) const
 {
-  return "not implemented";
+  return "Eval not implemented";
 }
 
 const std::string gpsshogi::
 Eval::describeAll(const std::string& feature) const
 {
-  return "not implemented";
+  return "Eval::describeAll not implemented";
 }
 
 boost::tuple<std::string, int, int> 
