@@ -113,94 +113,97 @@ OpenMidEndingBase::findFeature(size_t index) const
   return boost::make_tuple("not found", 0, 0);
 }
 
-class gpsshogi::OpenMidEndingBase::Stack : public EvalValueStack
+namespace gpsshogi
 {
-  const OpenMidEndingBase *eval;
-  enum { OthersLimit = 128 };
-  struct Entry
+  template <class Eval>
+  struct OMDStackBase : public EvalValueStack
   {
-    int sum;
-    progress::ml::NewProgress progress;
-    CArray<int,OthersLimit> flat;
-    CArray<MultiInt,OthersLimit> legacy;
-    CArray<MultiInt ,OthersLimit> triple;
-    CArray<CArray<MultiInt,2> ,OthersLimit> legacy_state;
-    CArray<CArray<MultiInt,2> ,OthersLimit> triple_state;
-    Entry() : sum(0), progress(NumEffectState()) {}
-  };
-  osl::stack<Entry> values;
-public:  
-  Stack(OpenMidEndingBase *ev, const NumEffectState& state) : eval(ev) { 
-    assert((int)eval->flat.size() <= OthersLimit);
-    assert((int)eval->legacy.size() <= OthersLimit);
-    assert((int)eval->triple.size() <= OthersLimit);
-    reset(state);
-  }
-  void push(const NumEffectState& new_state, Move moved)
-  {
-    Entry e = values.top();
-    e.sum = 0;
-    e.progress.update(new_state, moved);
-    const int progress = eval->progress(e.progress.progress());
-    for (size_t i=0; i<eval->flat.size(); ++i)
+    const Eval *eval;
+    enum { OthersLimit = 128 };
+    struct Entry
     {
-      if (const EvalProgress *p = dynamic_cast<const EvalProgress*>(&(eval->flat[i])))
-	e.flat[i] = p->eval(new_state, e.progress);
-      else
-	e.flat[i] = eval->flat[i].evalWithUpdate(new_state, moved, e.flat[i]);
+      int sum;
+      progress::ml::NewProgress progress;
+      CArray<int,OthersLimit> flat;
+      CArray<MultiInt,OthersLimit> legacy;
+      CArray<MultiInt ,OthersLimit> triple;
+      CArray<CArray<MultiInt,2> ,OthersLimit> legacy_state;
+      CArray<CArray<MultiInt,2> ,OthersLimit> triple_state;
+      Entry() : sum(0), progress(NumEffectState()) {}
+    };
+    osl::stack<Entry> values;
+    OMDStackBase(Eval *ev, const NumEffectState& state) : eval(ev) { 
+      assert((int)eval->flat.size() <= OthersLimit);
+      assert((int)eval->legacy.size() <= OthersLimit);
+      assert((int)eval->triple.size() <= OthersLimit);
+      reset(state);
     }
-    MultiInt multi_sum;
-    for (size_t i=0; i<eval->legacy.size(); ++i) {
-      e.legacy[i] = eval->legacy[i].evalWithUpdate(new_state, moved, e.legacy[i], eval->legacy_weights[i], e.legacy_state[i]);
-      multi_sum += e.legacy[i];
+    void push(const NumEffectState& new_state, Move moved)
+    {
+      Entry e = values.top();
+      e.sum = 0;
+      e.progress.update(new_state, moved);
+      const int progress = eval->progress(e.progress.progress());
+      for (size_t i=0; i<eval->flat.size(); ++i)
+      {
+	if (const EvalProgress *p = dynamic_cast<const EvalProgress*>(&(eval->flat[i])))
+	  e.flat[i] = p->eval(new_state, e.progress);
+	else
+	  e.flat[i] = eval->flat[i].evalWithUpdate(new_state, moved, e.flat[i]);
+      }
+      MultiInt multi_sum;
+      for (size_t i=0; i<eval->legacy.size(); ++i) {
+	e.legacy[i] = eval->legacy[i].evalWithUpdate(new_state, moved, e.legacy[i], eval->legacy_weights[i], e.legacy_state[i]);
+	multi_sum += e.legacy[i];
+      }
+      for (size_t i=0; i<eval->triple.size(); ++i) {
+	e.triple[i] = eval->triple[i].evalWithUpdateMulti(new_state, moved, e.triple[i], e.triple_state[i]);
+	multi_sum += e.triple[i];
+      }
+      // sum
+      for (size_t i=0; i<eval->flat.size(); ++i)
+	e.sum += e.flat[i];
+      e.sum = eval->compose(e.sum, multi_sum, progress);
+      values.push(e);
     }
-    for (size_t i=0; i<eval->triple.size(); ++i) {
-      e.triple[i] = eval->triple[i].evalWithUpdateMulti(new_state, moved, e.triple[i], e.triple_state[i]);
-      multi_sum += e.triple[i];
+    void pop() { assert(! values.empty()); values.pop(); }
+    int value() const { assert(! values.empty()); return values.top().sum; }
+    Progress16 progress16() const {
+      assert(! values.empty());
+      return values.top().progress.progress16();
     }
-    // sum
-    for (size_t i=0; i<eval->flat.size(); ++i)
-      e.sum += e.flat[i];
-    e.sum = eval->compose(e.sum, multi_sum, progress);
-    values.push(e);
-  }
-  void pop() { assert(! values.empty()); values.pop(); }
-  int value() const { assert(! values.empty()); return values.top().sum; }
-  Progress16 progress16() const {
-    assert(! values.empty());
-    return values.top().progress.progress16();
-  }
-  void reset(const NumEffectState& state)
-  {
-    while(! values.empty())
-      pop();
-    Entry e;
-    e.sum = 0;
-    e.progress = progress::ml::NewProgress(state);
-    const int progress = eval->progress(e.progress.progress());
-    for (size_t i=0; i<eval->flat.size(); ++i)
-      e.flat[i] = eval->flat[i].eval(state);
-    MultiInt multi_sum;
-    for (size_t i=0; i<eval->legacy.size(); ++i) {
-      e.legacy[i] = eval->legacy[i].eval(state, eval->legacy_weights[i], e.legacy_state[i]);
-      multi_sum += e.legacy[i];
+    void reset(const NumEffectState& state)
+    {
+      while(! values.empty())
+	pop();
+      Entry e;
+      e.sum = 0;
+      e.progress = progress::ml::NewProgress(state);
+      const int progress = eval->progress(e.progress.progress());
+      for (size_t i=0; i<eval->flat.size(); ++i)
+	e.flat[i] = eval->flat[i].eval(state);
+      MultiInt multi_sum;
+      for (size_t i=0; i<eval->legacy.size(); ++i) {
+	e.legacy[i] = eval->legacy[i].eval(state, eval->legacy_weights[i], e.legacy_state[i]);
+	multi_sum += e.legacy[i];
+      }
+      for (size_t i=0; i<eval->triple.size(); ++i) {
+	e.triple[i] = eval->triple[i].evalMulti(state, e.triple_state[i]);
+	multi_sum += e.triple[i];
+      }
+      // sum
+      for (size_t i=0; i<eval->flat.size(); ++i)
+	e.sum += e.flat[i];
+      e.sum = eval->compose(e.sum, multi_sum, progress);
+      values.push(e);
     }
-    for (size_t i=0; i<eval->triple.size(); ++i) {
-      e.triple[i] = eval->triple[i].evalMulti(state, e.triple_state[i]);
-      multi_sum += e.triple[i];
-    }
-    // sum
-    for (size_t i=0; i<eval->flat.size(); ++i)
-      e.sum += e.flat[i];
-    e.sum = eval->compose(e.sum, multi_sum, progress);
-    values.push(e);
-  }
-};
+  };
+}
 
 gpsshogi::EvalValueStack * gpsshogi::
 OpenMidEndingBase::newStack(const NumEffectState& state)
 {
-  return new Stack(this, state);
+  return new OMDStackBase<OpenMidEndingBase>(this, state);
 }
 
 int gpsshogi::
@@ -661,7 +664,7 @@ OpenMidEndingBase::describe(const std::string& feature, size_t local_index) cons
   for (size_t i=0; i<legacy.size(); ++i) {
     if (legacy[i].name() == feature) {
       const int dim = legacy[i].dimensionOne();
-      if (local_index * EvalStages >= dim)
+      if (local_index >= dim * EvalStages)
 	return "index error";
       const int value = legacy_weights[i].value(local_index%dim)
 	[local_index/dim];	  
@@ -869,6 +872,8 @@ KOpenMidEnding() : OpenMidEndingBase(true)
   triple.push_back(new EvalComponentStages(new RookSilverKnight));
   triple.push_back(new EvalComponentStages(new BishopSilverKnight));
   triple.push_back(new EvalComponentStages(new AttackMajorsInBase));
+  triple.push_back(new EvalComponentStages(new CheckShadowPtype));
+  triple.push_back(new EvalComponentStages(new Promotion37));
   addFinished();
 }
 
@@ -1017,6 +1022,8 @@ StableOpenMidEnding() : OpenMidEndingBase(false)
   triple.push_back(new EvalComponentStages(new RookSilverKnight));
   triple.push_back(new EvalComponentStages(new BishopSilverKnight));
   triple.push_back(new EvalComponentStages(new AttackMajorsInBase));
+  triple.push_back(new EvalComponentStages(new CheckShadowPtype));
+  triple.push_back(new EvalComponentStages(new Promotion37));
   addFinished();
 }
 
