@@ -296,7 +296,7 @@ CanCheckNonPawnPieceStandCombinationEach::featuresPlayer(
 {
   const Player defense = PlayerTraits<P>::opponent;
   const King8Info king = state.king8Info(defense);
-  if (!KnightCheck::canCheck(state, state.getKingPiece<defense>()) &&
+  if (!KnightCheck::canCheck(state, state.kingPiece<defense>()) &&
       king.dropCandidate() == 0 && king.hasMoveCandidate<P>(state) == 0)
     return;
 
@@ -391,7 +391,7 @@ CanCheckNonPawnPieceStandCombination::featuresOneNonUniq(
   int black_index = -1;
 
   const King8Info white_king = state.king8Info(WHITE);
-  if (KnightCheck::canCheck(state, state.getKingPiece<WHITE>()) ||
+  if (KnightCheck::canCheck(state, state.kingPiece<WHITE>()) ||
       white_king.dropCandidate() != 0 || white_king.hasMoveCandidate<BLACK>(state))
   {
     black_index = index(state.countPiecesOnStand<ROOK>(BLACK),
@@ -403,7 +403,7 @@ CanCheckNonPawnPieceStandCombination::featuresOneNonUniq(
   }
   int white_index = -1;
   const King8Info black_king = state.king8Info(BLACK);
-  if (KnightCheck::canCheck(state, state.getKingPiece<BLACK>()) ||
+  if (KnightCheck::canCheck(state, state.kingPiece<BLACK>()) ||
       black_king.dropCandidate() != 0 || black_king.hasMoveCandidate<WHITE>(state))
   {
     white_index = index(state.countPiecesOnStand<ROOK>(WHITE),
@@ -438,11 +438,11 @@ NonPawnPieceStandTurn::featuresOneNonUniq(
       state.countPiecesOnStand(osl::WHITE, osl::PieceStand::order[i]);
     for (int j = 0; j < black_count; ++j)
     {
-      feature_count.add(index(BLACK, state.getTurn(), ptype, j), 1);
+      feature_count.add(index(BLACK, state.turn(), ptype, j), 1);
     }
     for (int j = 0; j < white_count; ++j)
     {
-      feature_count.add(index(WHITE, state.getTurn(), ptype, j), -1);
+      feature_count.add(index(WHITE, state.turn(), ptype, j), -1);
     }
   }
 }
@@ -485,9 +485,9 @@ void gpsshogi::PieceStandY::featuresOneNonUniq(
   const NumEffectState &state,
   IndexCacheI<MaxActiveWithDuplication> &features) const
 {
-  const CArray<Position,2> kings = {{ 
-      state.getKingPosition(BLACK),
-      state.getKingPosition(WHITE),
+  const CArray<Square,2> kings = {{ 
+      state.kingSquare(BLACK),
+      state.kingSquare(WHITE),
     }};
   for (size_t i = 0; i < osl::PieceStand::order.size(); ++i)
   {
@@ -587,7 +587,7 @@ void gpsshogi::PieceStandOnBoard::featuresOneNonUniq(
 {
   for (int i = 0; i < Piece::SIZE; ++i)
   {
-    const Piece piece = state.getPieceOf(i);
+    const Piece piece = state.pieceOf(i);
     if (!piece.isOnBoard())
     {
       continue;
