@@ -1,11 +1,16 @@
 /* progressFeature.cc
  */
 #include "eval/progressFeature.h"
+#include "eval/openMidEnding.h"
 #include "osl/eval/ml/openMidEndingEval.h"
-#include "osl/stl/stack.h"
 #include "osl/progress/effect5x3.h"
+#include "osl/effect_util/additionalEffect.h"
+#include "osl/stl/stack.h"
 #include "osl/container/pieceVector.h"
 #include "osl/pieceStand.h"
+#include "osl/record/csa.h"
+#include <boost/lexical_cast.hpp>
+#include <boost/foreach.hpp>
 #include <functional>
 #include <numeric>
 #include <fstream>
@@ -16,7 +21,7 @@ typedef osl::eval::ml::OpenMidEndingEval ome_eval_t;
 
 gpsshogi::
 ProgressFeatureBase::ProgressFeatureBase()
-  : max_progress(32), scale(1.0)
+  : max_progress(1023), scale(1.0)
 {
   assert(ome_eval_t::initialized());
   static bool initialized = false;
@@ -84,9 +89,7 @@ public:
     int max = eval->maxProgress();
     e.eval_value = compose(progress, max,
 			   e.test_eval.progressIndependentValue(),
-			   e.test_eval.openingValue(),
-			   e.test_eval.midgameValue(),
-			   e.test_eval.endgameValue());
+			   e.test_eval.stageValue());
     values.push(e);
   }
   void pop() { assert(! values.empty()); values.pop(); }
@@ -104,9 +107,7 @@ public:
     int max = eval->maxProgress();
     e.eval_value = compose(progress, max,
 			   e.test_eval.progressIndependentValue(),
-			   e.test_eval.openingValue(),
-			   e.test_eval.midgameValue(),
-			   e.test_eval.endgameValue());
+			   e.test_eval.stageValue());
     values.push(e);
   }
 };
@@ -146,9 +147,7 @@ ProgressFeatureBase::eval(const NumEffectState& state) const
   }
   return compose(progress, maxProgress(),
 		 eval.progressIndependentValue(),
-		 eval.openingValue(),
-		 eval.midgameValue(),
-		 eval.endgameValue());
+		 eval.stageValue());
 }
 
 bool gpsshogi::
@@ -221,10 +220,20 @@ ProgressFeatureBase::saveWeight(double *w) const
   w[dimension()-1] = max_progress;
 }
 
+int gpsshogi::
+ProgressFeatureBase::compose(int progress, int progress_max,
+			     int flat, const MultiInt& stage_value)
+{
+  if (progress_max == 1024)
+    progress_max = 1023;	// xxx
+  return StableOpenMidEnding::compose
+    (flat, stage_value, progress, progress_max, false);
+}
+
 void gpsshogi::
 ProgressFeatureBase::features(const NumEffectState& state, double& value,
-			   vector<std::pair<int, double> >& out,
-			   int offset) const
+			      vector<std::pair<int, double> >& out,
+			      int offset) const
 {
   ome_eval_t eval(state);
   int dim = 0, progress = 0;
@@ -234,36 +243,34 @@ ProgressFeatureBase::features(const NumEffectState& state, double& value,
     progress += his_value;
     dim += all[i].dimension();
   }
-  const int max = maxProgress();
-  if (progress <= 0 || progress >= max) {
-    out.clear();
-    if (progress <= 0)
-      out.push_back(std::make_pair(dimension()-1, eval.openingValue()+eval.progressIndependentValue()));
-    else
-      out.push_back(std::make_pair(dimension()-1, eval.endgameValue()+eval.progressIndependentValue()));
+  const int progress_max = maxProgress();
+  const int flat_value = eval.progressIndependentValue();
+  const MultiInt stage_value = eval.stageValue();
+  if (progress < 0)
+    progress = 0;
+  if (progress >= progress_max) 
+    progress = progress_max-1;
+
+  const int c0 = progress_max/3, c1 = c0*2;
+  if (progress < c0) {
+    const double diff = stage_value[1] - stage_value[0];
+    for (size_t i = 0; i < out.size(); ++i)
+      out[i].second *= diff;
+    out.push_back(std::make_pair(dimension()-1, (stage_value[0]+flat_value)/3.0));
+  } else if (progress < c1) {
+    const double diff = stage_value[2] - stage_value[1];
+    for (size_t i = 0; i < out.size(); ++i)
+      out[i].second *= diff;
+    out.push_back(std::make_pair(dimension()-1, (2*stage_value[1]-stage_value[2]+flat_value)/3.0));
   } else {
-    const int c = max/2;
-    if (progress == c) {
-      out.clear();
-      out.push_back(std::make_pair(dimension()-1, eval.midgameValue()+eval.progressIndependentValue()));
-    } else if (progress < c) {
-      const double scale = eval.midgameValue() - eval.openingValue();
-      for (size_t i = 0; i < out.size(); ++i)
-	out[i].second *= 2*scale;
-      out.push_back(std::make_pair(dimension()-1, eval.openingValue()+eval.progressIndependentValue()));
-    } else {
-      assert(c < progress && c < max);
-      const double scale = eval.endgameValue() - eval.midgameValue();
-      for (size_t i = 0; i < out.size(); ++i)
-	out[i].second *= 2*scale;
-      out.push_back(std::make_pair(dimension()-1, eval.midgameValue()*2 - eval.endgameValue() +eval.progressIndependentValue()));      
-    }
+    assert(c1 <= progress && c1 < progress_max);
+    const double diff = stage_value[3] - stage_value[2];
+    for (size_t i = 0; i < out.size(); ++i)
+      out[i].second *= diff;
+    out.push_back(std::make_pair(dimension()-1, (stage_value[2]*3-stage_value[3]*2 +flat_value)/3.0));
   }
-  value = compose(progress, max, 
-		  eval.progressIndependentValue(),
-		  eval.openingValue(),
-		  eval.midgameValue(),
-		  eval.endgameValue());
+
+  value = compose(progress, progress_max, flat_value, stage_value);
 }
 
 void gpsshogi::
@@ -286,7 +293,8 @@ void gpsshogi::
 ProgressFeatureBase::setRandom()
 {
   for (size_t i=0; i<all.size(); ++i)
-    all[i].setRandom();
+    for (size_t j=0; j<all[i].dimension(); ++j)
+      all[i].setValue(j, osl::random()%4);
 }
 
 boost::tuple<std::string, int, int> 
@@ -300,6 +308,41 @@ gpsshogi::ProgressFeatureBase::findFeature(size_t index) const
   return boost::make_tuple("max_progress", index, 1);
 }
 
+const std::string gpsshogi::
+ProgressFeatureBase::describe(const std::string& feature, size_t local_index) const
+{
+  for (size_t i=0; i<all.size(); ++i) {
+    if (all[i].name() == feature) {
+      if (local_index >= all[i].dimension())
+	return "index error";
+      return all[i].describe(local_index);
+    }
+  }
+  if (feature == "max_progress")
+    return "max_progress";
+  return "not found";
+}
+
+const std::string gpsshogi::
+ProgressFeatureBase::describeAll(const std::string& feature) const
+{
+  for (size_t i=0; i<all.size(); ++i) {
+    if (all[i].name() == feature) {
+      std::string out;
+      for (size_t j=0; j<all[i].dimension(); ++j) {
+	int value = all[i].value(j);
+	if (value != 0)
+	  out += all[i].describe(j)
+	    + " value " + boost::lexical_cast<std::string>(value) + "\n";
+      }
+      return out;
+    }
+  }
+  if (feature == "max_progress")
+    return "max_progress";
+  return "not found";
+}
+
 /* ------------------------------------------------------------------------- */
 
 gpsshogi::NullProgressFeatureEval::NullProgressFeatureEval()
@@ -383,6 +426,7 @@ namespace gpsshogi
 			 index_list_t&diffs, int offset) const;
     void showSummary(std::ostream &os) const;
     const std::string name() const { return "ProgressPieceStand"; };
+    const std::string describe(size_t local_index) const;
   };
 
   class ProgressPieceKingRelative : public EvalComponent
@@ -499,19 +543,22 @@ namespace gpsshogi
     ProgressNonPawnAttackedPtypePair()
       : EvalComponent((PTYPE_SIZE * 2 * PTYPE_SIZE)*(PTYPE_SIZE * 2 * PTYPE_SIZE))
     { 
+      for (size_t i = 0; i < dimension(); ++i)
+	values[i] = 1;
     }
     void countOne(PieceMask attacked, const NumEffectState& state,
 		  index_list_t &features, int offset) const
     {
-      // ɾؿѤȤΰ㤤ϡ֤
+      // 評価関数用との違いは、後手番の符号
       PieceVector pieces;
       while (attacked.any())
       {
 	const Piece piece = state.pieceOf(attacked.takeOneBit());
 	pieces.push_back(piece);
       }
-      for (size_t i=0; i+1<pieces.size(); ++i) {
+      for (size_t i=0; i<pieces.size(); ++i) {
 	const int i0 = index1(state, pieces[i]);
+	features.add(index2(0, i0) + offset, 1);	
 	for (size_t j=i+1; j<pieces.size(); ++j) {
 	  const int i1 = index1(state, pieces[j]);
 	  features.add(index2(i0, i1) + offset, 1);
@@ -538,7 +585,7 @@ namespace gpsshogi
     const std::string name() const { return "ProgressNonPawnAttackedPtypePair"; }
     size_t maxActive() const { return 40*40*2; }
 
-    // ʣ
+    // 重複
     int eval(const NumEffectState &state) const
     {
       index_list_t values;
@@ -550,6 +597,12 @@ namespace gpsshogi
       }
       return result;
     }
+    const std::string describe(size_t local_index) const
+    {
+      const int i0 = local_index / (PTYPE_SIZE * 2 * PTYPE_SIZE);
+      const int i1 = local_index % (PTYPE_SIZE * 2 * PTYPE_SIZE);
+      return describeOne(i0) + " " + describeOne(i1);
+    }
   private:
     static int index1(const NumEffectState &state, Piece piece)
     {
@@ -566,8 +619,197 @@ namespace gpsshogi
 	std::swap(i0, i1);
       return i0 * PTYPE_SIZE * 2 * PTYPE_SIZE + i1;
     }
+    const std::string describeOne(size_t index) const
+    {
+      if (index == 0)
+	return "--";
+      const Ptype attack = static_cast<Ptype>(index%PTYPE_SIZE);
+      index /= PTYPE_SIZE;
+      const Ptype ptype = static_cast<Ptype>(index%PTYPE_SIZE);
+      const bool has_support = index > PTYPE_SIZE;
+      return record::csa::show(attack) + "=>" + record::csa::show(ptype)
+	+ (has_support ? "#" : ".");
+    }
   };
 
+  // 
+  class ProgressPawnFacing : public EvalComponent
+  {
+  public:
+    ProgressPawnFacing() : EvalComponent(10)
+    { 
+      for (size_t i = 0; i < dimension(); ++i)
+	values[i] = 1;
+    }
+    void featuresNonUniq(const NumEffectState &state,
+			 index_list_t &features, int offset) const
+    {
+      PieceMask attacked = state.effectedMask(WHITE) & state.piecesOnBoard(BLACK);
+      mask_t pawn = attacked.selectBit<PAWN>()
+	& ~(state.promotedPieces().getMask<PAWN>());
+      int count = 0;
+      while (pawn.any()) {
+	const Piece p(state.pieceOf(pawn.takeOneBit()+PtypeFuns<PAWN>::indexNum*32));
+	if (state.hasEffectByPtypeStrict<PAWN>(WHITE, p.square()))
+	  ++count;
+      }
+      if (count == 0)
+	features.add(offset, 1);
+      for (int i=1; i<=count; ++i)
+	features.add(offset+i, 1);
+    }
+    const std::string name() const { return "ProgressPawnFacing"; }
+    size_t maxActive() const { return 9; }
+
+    int eval(const NumEffectState &state) const
+    {
+      index_list_t values;
+      int result = 0;
+      featuresNonUniq(state, values, 0);
+      for (size_t i = 0; i < values.size(); ++i)
+      {
+	result += this->value(values[i].first) * values[i].second;
+      }
+      return result;
+    }
+    void showSummary(std::ostream &os) const
+    {
+      os << "PawnFacing ";
+      for (size_t i = 0; i < dimension(); ++i)
+	os << this->value(i) << " ";
+      os << std::endl;
+    }
+  };
+  // 
+  class ProgressPromotion37 : public EvalComponent
+  {
+  public:
+    ProgressPromotion37() : EvalComponent(PTYPE_SIZE)
+    { 
+      for (size_t i = 0; i < dimension(); ++i)
+	values[i] = 1;
+    }
+    void addFeatures(const NumEffectState &state,
+		     index_list_t &features, int offset,
+		     Player attack, int rank) const
+    {
+      CArray<int,PTYPE_SIZE> count = {{ 0 }};
+      for (int x=1; x<=9; ++x) {
+	const Square target(x, rank);
+	if (! state[target].isEmpty())
+	  continue;
+	int a = state.countEffect(attack, target);
+	const int d = state.countEffect(alt(attack), target);
+	if (a > 0 && a == d)
+	  a += AdditionalEffect::hasEffect(state, target, attack);
+	if (a < d)
+	  continue;
+	const Ptype ptype = state.findCheapAttack(attack, target).ptype();
+	if (isPiece(ptype) && ! isPromoted(ptype))
+	  count[ptype]++;
+      }
+      for (int p=PTYPE_BASIC_MIN; p<=PTYPE_MAX; ++p) {
+	if (count[p] > 0)
+	  features.add(offset+p, 1);
+	if (count[p] > 1)
+	  features.add(offset+p-8, (count[p]-1));
+      }
+    }
+    void featuresNonUniq(const NumEffectState &state,
+			 index_list_t &features, int offset) const
+    {
+      // black
+      addFeatures(state, features, offset, BLACK, 3);
+      // white
+      addFeatures(state, features, offset, WHITE, 7);
+    }
+    const std::string name() const { return "ProgressPromotion37"; }
+    size_t maxActive() const { return PTYPE_SIZE*2; }
+
+    int eval(const NumEffectState &state) const
+    {
+      index_list_t values;
+      int result = 0;
+      featuresNonUniq(state, values, 0);
+      for (size_t i = 0; i < values.size(); ++i)
+      {
+	result += this->value(values[i].first) * values[i].second;
+      }
+      return result;
+    }
+    void showSummary(std::ostream &os) const
+    {
+      os << "ProgressPromotion37 ";
+      for (size_t i = 0; i < dimension(); ++i)
+	os << this->value(i) << " ";
+      os << std::endl;
+    }
+  };
+  // 
+  class ProgressPieceStand7 : public EvalComponent
+  {
+  public:
+    enum { StandRank = 8 };
+    ProgressPieceStand7() : EvalComponent(StandRank*7)
+    { 
+      for (size_t i = 0; i < dimension(); ++i)
+	values[i] = 1;
+    }
+    void addFeatures(const NumEffectState &state,
+		     index_list_t &features, int offset,
+		     Player attack) const
+    {
+      CArray<int,7> stand = {{ 0 }};
+      int filled = 0;
+      BOOST_FOREACH(Ptype ptype, PieceStand::order)
+	if (state.hasPieceOnStand(attack, ptype))
+	  stand[filled++] = ptype-PTYPE_BASIC_MIN;
+      for (int i=0; i<std::min(7,filled+1); ++i) {
+	assert(stand[i] < StandRank);
+	features.add(offset+(stand[i] + StandRank*i), 1);
+      }
+    }
+    void featuresNonUniq(const NumEffectState &state,
+			 index_list_t &features, int offset) const
+    {
+      // black
+      addFeatures(state, features, offset, BLACK);
+      // white
+      addFeatures(state, features, offset, WHITE);
+    }
+    const std::string name() const { return "ProgressPieceStand7"; }
+    size_t maxActive() const { return 7*2; }
+
+    int eval(const NumEffectState &state) const
+    {
+      index_list_t values;
+      int result = 0;
+      featuresNonUniq(state, values, 0);
+      for (size_t i = 0; i < values.size(); ++i)
+      {
+	result += this->value(values[i].first) * values[i].second;
+      }
+      return result;
+    }
+    void showSummary(std::ostream &os) const
+    {
+      os << "ProgressPieceStand7\n";
+      for (int i=0; i<7; ++i) {
+	BOOST_FOREACH(Ptype ptype, PieceStand::order)
+	  os << "  " << record::csa::show(ptype) << ' '
+	     << this->value(ptype-PTYPE_BASIC_MIN + StandRank*i);
+	os << "\n";
+      }
+    }
+    const std::string describeOne(size_t index) const
+    {
+      Ptype ptype = static_cast<Ptype>((index%StandRank)+PTYPE_PIECE_MIN);
+      if (ptype == KING)
+	ptype = PTYPE_EMPTY;
+      return record::csa::show(ptype)
+	+ "(" + static_cast<char>('0'+(index/StandRank)) + ")";
+    }
+  };
 }
 
 int gpsshogi::ProgressPieceStand::evalWithUpdate(
@@ -617,6 +859,20 @@ int gpsshogi::ProgressPieceStand::eval(
   return result;
 }
 
+const std::string gpsshogi::
+ProgressPieceStand::describe(size_t local_index) const
+{
+  for (size_t i = 0; i < osl::PieceStand::order.size(); ++i)
+  {
+    const Ptype ptype = osl::PieceStand::order[i];
+    if (local_index >= Ptype_Table.getIndexMin(ptype)
+	&& local_index < Ptype_Table.getIndexLimit(ptype))
+      return Ptype_Table.getCsaName(ptype)
+	+ boost::lexical_cast<std::string>(local_index - Ptype_Table.getIndexMin(ptype));
+  }
+  return "ProgressPieceStand error";
+}
+
 void gpsshogi::ProgressPieceStand::featuresNonUniq(
   const osl::state::NumEffectState &state, 
   index_list_t&diffs,
@@ -669,12 +925,15 @@ gpsshogi::EffectProgressFeatureEval::EffectProgressFeatureEval()
   all.push_back(new Effect5x5Features);
   all.push_back(new ProgressPieceKingRelative);
   all.push_back(new ProgressNonPawnAttackedPtypePair);
+  all.push_back(new ProgressPawnFacing);
+  all.push_back(new ProgressPromotion37);
+  all.push_back(new ProgressPieceStand7);
   addFinished();
 }
 
 double gpsshogi::EffectProgressFeatureEval::maxProgressConstraint() const
 {
-  return 1024;
+  return 1023;
 }
 #endif
 gpsshogi::StableEffectProgressFeatureEval::StableEffectProgressFeatureEval()
@@ -684,12 +943,15 @@ gpsshogi::StableEffectProgressFeatureEval::StableEffectProgressFeatureEval()
   all.push_back(new Effect5x5Features);
   all.push_back(new ProgressPieceKingRelative);
   all.push_back(new ProgressNonPawnAttackedPtypePair);
+  all.push_back(new ProgressPawnFacing);
+  all.push_back(new ProgressPromotion37);
+  all.push_back(new ProgressPieceStand7);
   addFinished();
 }
 
 double gpsshogi::StableEffectProgressFeatureEval::maxProgressConstraint() const
 {
-  return 1024;
+  return 1023;
 }
 
 void gpsshogi::
