@@ -32,6 +32,7 @@ namespace gpsshogi
     bool load(const char *filename);
     void save(const char *filename) const;
     void setWeight(const double*);
+    void setWeightScale(const double*, double);
     void saveWeight(double*) const;
 
     int flatValue(size_t index) const {
@@ -49,6 +50,7 @@ namespace gpsshogi
     void showSummary(std::ostream&) const;
     void showAll(std::ostream&) const;
     void setRandom();
+    int roundUp() const { return 1; }
 
     size_t maxActive() const;
     int pawnValue() const;
@@ -59,23 +61,9 @@ namespace gpsshogi
     class Stack;
     friend class Stack;
     EvalValueStack *newStack(const NumEffectState& state);
-    static int compose(int progress, int progress_max, int independent, int opening, int midgame, int endgame)
-    {
-      const int c = progress_max/2;
-      progress = std::max(std::min(progress, progress_max), 0);
-      int sum = independent * progress_max;
-      if (progress < c) 
-      {
-	sum += opening * 2*(c - progress);
-	sum += midgame * 2*progress;
-      }
-      else 
-      {
-	sum += midgame * 2*(progress_max - progress);
-	sum += endgame * 2*(progress - c);
-      }
-      return sum;
-    }
+    static int compose(int progress, int progress_max, int independent, const MultiInt& stage_value);
+    const std::string describe(const std::string& feature, size_t local_index) const;
+    const std::string describeAll(const std::string& feature) const;
   };
   
   class NullProgressFeatureEval : public ProgressFeatureBase
@@ -135,7 +123,7 @@ namespace gpsshogi
     template <class Null5x3Type, Player P>
     static void featuresEach(
       Null5x3Type, Player2Type<P>,
-      Position, Position,
+      Square, Square,
       int, int,
       int,
       IndexCacheI<MaxActiveWithDuplication> &)
@@ -145,7 +133,7 @@ namespace gpsshogi
     template <class Head, class Tail, Player P>
     static void featuresEach(
       Feature5x3List<Head, Tail>, Player2Type<P>,
-      Position king, Position target,
+      Square king, Square target,
       int attack_count, int defense_count,
       int offset,
       IndexCacheI<MaxActiveWithDuplication> &out)
@@ -162,8 +150,8 @@ namespace gpsshogi
 			  int offset,
 			  IndexCacheI<MaxActiveWithDuplication> &out)
     {
-      const Position king = state.getKingPosition<Defense>();
-      const Position center = Centering5x3::adjustCenter(king);
+      const Square king = state.kingSquare<Defense>();
+      const Square center = Centering5x3::adjustCenter(king);
       const int min_x = center.x() - 2;
       const int min_y = center.y() - 1;
 
@@ -171,8 +159,8 @@ namespace gpsshogi
       {
 	for (int dy=0; dy<3; ++dy)
 	{
-	  const Position target(min_x+dx,min_y+dy);
-	  const NumBitmapEffect effect = state.getEffect(target);
+	  const Square target(min_x+dx,min_y+dy);
+	  const NumBitmapEffect effect = state.effectSetAt(target);
 	  const int attack_count =
 	    effect.countEffect(PlayerTraits<Defense>::opponent);
 	  const int defense_count = effect.countEffect(Defense);
@@ -197,7 +185,7 @@ namespace gpsshogi
   {
   private:
     template <Player P>
-    static int index(Position king, Position target)
+    static int index(Square king, Square target)
     {
       const int x_diff = std::abs(king.x() - target.x()); // [0, 4]
       const int y_diff = (P == BLACK ? king.y() - target.y() :
@@ -207,7 +195,7 @@ namespace gpsshogi
   public:
     enum { DIM = 25 };
     template <Player Defense>
-    static void features(Position king, Position target,
+    static void features(Square king, Square target,
 			 int attack_count, int /*defense_count*/,
 			 int offset,
 			 IndexCacheI<MaxActiveWithDuplication> &out)
@@ -223,7 +211,7 @@ namespace gpsshogi
   {
   private:
     template <Player P>
-    static int index(Position king, Position target)
+    static int index(Square king, Square target)
     {
       const int x_diff = std::abs(king.x() - target.x()); // [0, 4]
       const int y_diff = (P == BLACK ? king.y() - target.y() :
@@ -233,7 +221,7 @@ namespace gpsshogi
   public:
     enum { DIM = 25 };
     template <Player Defense>
-    static void features(Position king, Position target,
+    static void features(Square king, Square target,
 			 int /*attack_count*/, int defense_count,
 			 int offset,
 			 IndexCacheI<MaxActiveWithDuplication> &out)
@@ -249,7 +237,7 @@ namespace gpsshogi
   {
   private:
     template <Player P>
-    static int index(Position king, Position target)
+    static int index(Square king, Square target)
     {
       int target_x = (king.x() > 5 ? 10 - king.x() : king.x()); // [1, 5]
       int x_diff = king.x() - target.x(); // [-4, 4]
@@ -268,7 +256,7 @@ namespace gpsshogi
   public:
     enum { DIM = 225 };
     template <Player Defense>
-    static void features(Position king, Position target,
+    static void features(Square king, Square target,
 			 int attack_count, int /*defense_count*/,
 			 int offset,
 			 IndexCacheI<MaxActiveWithDuplication> &out)
@@ -284,7 +272,7 @@ namespace gpsshogi
   {
   private:
     template <Player P>
-    static int index(Position king, Position target)
+    static int index(Square king, Square target)
     {
       const int x_diff = std::abs(king.x() - target.x()); // [0, 4]
       const int y_diff = (P == BLACK ? king.y() - target.y() :
@@ -295,7 +283,7 @@ namespace gpsshogi
   public:
     enum { DIM = 225 };
     template <Player Defense>
-    static void features(Position king, Position target,
+    static void features(Square king, Square target,
 			 int attack_count, int /*defense_count*/,
 			 int offset,
 			 IndexCacheI<MaxActiveWithDuplication> &out)
@@ -311,7 +299,7 @@ namespace gpsshogi
   {
   private:
     template <Player P>
-    static int index(Position king, Position target,
+    static int index(Square king, Square target,
 		     int count)
     {
       const int x_diff = std::abs(king.x() - target.x()); // [0, 4]
@@ -322,7 +310,7 @@ namespace gpsshogi
   public:
     enum { DIM = 225 };
     template <Player Defense>
-    static void features(Position king, Position target,
+    static void features(Square king, Square target,
 			 int attack_count, int /*defense_count*/,
 			 int offset,
 			 IndexCacheI<MaxActiveWithDuplication> &out)
@@ -335,7 +323,7 @@ namespace gpsshogi
   {
   private:
     template <Player P>
-    static int index(Position king, Position target,
+    static int index(Square king, Square target,
 		     int count)
     {
       const int x_diff = std::abs(king.x() - target.x()); // [0, 4]
@@ -346,7 +334,7 @@ namespace gpsshogi
   public:
     enum { DIM = 225 };
     template <Player Defense>
-    static void features(Position king, Position target,
+    static void features(Square king, Square target,
 			 int /*attack_count*/, int defense_count,
 			 int offset,
 			 IndexCacheI<MaxActiveWithDuplication> &out)
@@ -359,7 +347,7 @@ namespace gpsshogi
   {
   private:
     template <Player P>
-    static int index(Position king, Position target,
+    static int index(Square king, Square target,
 		     int count)
     {
       const int king_y = (P == BLACK ? king.y() : 10 - king.y());
@@ -371,7 +359,7 @@ namespace gpsshogi
   public:
     enum { DIM = 2025 };
     template <Player Defense>
-    static void features(Position king, Position target,
+    static void features(Square king, Square target,
 			 int attack_count, int /*defense_count*/,
 			 int offset,
 			 IndexCacheI<MaxActiveWithDuplication> &out)
@@ -384,7 +372,7 @@ namespace gpsshogi
   {
   private:
     template <Player P>
-    static int index(Position king, Position target,
+    static int index(Square king, Square target,
 		     int count)
     {
       const int king_x = (king.x() > 5 ? 10 - king.x() : king.x());
@@ -400,7 +388,7 @@ namespace gpsshogi
   public:
     enum { DIM = 2025 };
     template <Player Defense>
-    static void features(Position king, Position target,
+    static void features(Square king, Square target,
 			 int attack_count, int /*defense_count*/,
 			 int offset,
 			 IndexCacheI<MaxActiveWithDuplication> &out)
@@ -478,7 +466,7 @@ namespace gpsshogi
     template <class Null5x5Type, Player P>
     static void featuresEach(
       Null5x5Type, Player2Type<P>,
-      Position /*king*/, Position /*target*/,
+      Square /*king*/, Square /*target*/,
       int /*attack_count*/, int /*defense_count*/,
       int /*offset*/,
       IndexCacheI<MaxActiveWithDuplication> &/*out*/)
@@ -488,12 +476,12 @@ namespace gpsshogi
     template <class Head, class Tail, Player P>
     static void featuresEach(
       Feature5x5List<Head, Tail>, Player2Type<P>,
-      Position king, Position target,
+      Square king, Square target,
       int attack_count, int defense_count,
       int offset,
       IndexCacheI<MaxActiveWithDuplication> &out)
     {
-      Head::template featuresPosition<P>(king, target,
+      Head::template featuresSquare<P>(king, target,
 					 attack_count, defense_count,
 					 offset, out);
       featuresEach(Tail(), Player2Type<P>(), king, target,
@@ -504,7 +492,7 @@ namespace gpsshogi
     static void featuresMask(
       Null5x5Type, Player2Type<Attack>,
       const NumEffectState &,
-      Position ,
+      Square ,
       const PieceMask &,
       int ,
       IndexCacheI<MaxActiveWithDuplication> &)
@@ -515,7 +503,7 @@ namespace gpsshogi
     static void featuresMask(
       Feature5x5List<Head, Tail>, Player2Type<P>,
       const NumEffectState &state,
-      Position king,
+      Square king,
       const PieceMask &king25,
       int offset,
       IndexCacheI<MaxActiveWithDuplication> &out)
@@ -530,8 +518,8 @@ namespace gpsshogi
 			  int offset,
 			  IndexCacheI<MaxActiveWithDuplication> &out)
     {
-      const Position king =
-	state.getKingPosition<PlayerTraits<Attack>::opponent>();
+      const Square king =
+	state.kingSquare<PlayerTraits<Attack>::opponent>();
       const int min_x = std::max(1, king.x() - 2);
       const int max_x = std::min(9, king.x() + 2);
       const int min_y = std::max(1, king.y() - 2);
@@ -542,8 +530,8 @@ namespace gpsshogi
       {
 	for (int x = min_x; x <= max_x; ++x)
 	{
-	  const Position target(x, y);
-	  const NumBitmapEffect effect = state.getEffect(target);
+	  const Square target(x, y);
+	  const NumBitmapEffect effect = state.effectSetAt(target);
 	  mask = mask | effect;
 	  const int attack_count = effect.countEffect(Attack);
 	  const int defense_count =
@@ -553,7 +541,7 @@ namespace gpsshogi
 		       offset, out);
 	}
       }
-      mask = mask & state.getOnBoardMask(Attack);
+      mask = mask & state.piecesOnBoard(Attack);
       featuresMask(Features(), Player2Type<Attack>(),
 		   state, king, mask, offset, out);
     }
@@ -571,7 +559,7 @@ namespace gpsshogi
   {
   public:
     template <Player Attack>
-    static void featuresPosition(Position /*king*/, Position /*target*/,
+    static void featuresSquare(Square /*king*/, Square /*target*/,
 				 int /*attack_count*/, int /*defense_count*/,
 				 int /*offset*/,
 				 IndexCacheI<MaxActiveWithDuplication> &/*out*/)
@@ -579,7 +567,7 @@ namespace gpsshogi
     }
     template <Player Attack>
     static void featuresMask(const NumEffectState &,
-			     Position ,
+			     Square ,
 			     const PieceMask &,
 			     int ,
 			     IndexCacheI<MaxActiveWithDuplication> &) { }
@@ -590,7 +578,7 @@ namespace gpsshogi
   public:
     enum { DIM = 75 };
     template <Player Attack>
-    static void featuresPosition(Position king, Position target,
+    static void featuresSquare(Square king, Square target,
 				 int attack_count, int defense_count,
 				 int offset,
 				 IndexCacheI<MaxActiveWithDuplication> &out)
@@ -612,7 +600,7 @@ namespace gpsshogi
     enum { DIM = 1125 };
     template <Player Attack>
     static void featuresMask(const NumEffectState &state,
-			     Position /*king*/,
+			     Square /*king*/,
 			     const PieceMask &king25,
 			     int offset,
 			     IndexCacheI<MaxActiveWithDuplication> &out)
@@ -646,7 +634,7 @@ namespace gpsshogi
     enum { DIM = 5625 };
     template <Player Attack>
     static void featuresMask(const NumEffectState &state,
-			     Position king,
+			     Square king,
 			     const PieceMask &king25,
 			     int offset,
 			     IndexCacheI<MaxActiveWithDuplication> &out)
@@ -685,7 +673,7 @@ namespace gpsshogi
     enum { DIM = 10125 };
     template <Player Attack>
     static void featuresMask(const NumEffectState &state,
-			     Position king,
+			     Square king,
 			     const PieceMask &king25,
 			     int offset,
 			     IndexCacheI<MaxActiveWithDuplication> &out)
