@@ -7,9 +7,11 @@
 #include "osl/misc/fixedCapacityVector.h"
 namespace gpsshogi
 {
-  enum { PvMaxDepth = 32 };
+  enum { PvMaxDepth = 48 };
   struct PVVector : public osl::FixedCapacityVector<osl::Move,PvMaxDepth> 
   {
+    int pieceValue() const;
+    int pieceValueOfTurn() const;
   };
   std::ostream& operator<<(std::ostream&, const PVVector&);
 }
