@@ -4,11 +4,16 @@
 #include "osl/search/sortCaptureMoves.h"
 #include "osl/search/shouldPromoteCut.h"
 #include "osl/search/quiescenceGenerator.h"
+#include "osl/move_probability/featureSet.h"
+#include "osl/rating/featureSet.h"
+#include "osl/rating/ratingEnv.h"
+#include "osl/record/opening/bookInMemory.h"
 #include "osl/move_order/captureSort.h"
 #include "osl/checkmate/immediateCheckmate.h"
 #include "osl/eval/evalTraits.h"
 #include "osl/eval/pieceEval.h"
 #include "osl/effect_util/effectUtil.h"
+#include "osl/move_generator/legalMoves.h"
 #include "osl/move_generator/capture_.h"
 #include "osl/move_generator/promote_.h"
 #include "osl/move_generator/escape_.h"
@@ -16,7 +21,9 @@
 #include "osl/move_generator/allMoves.h"
 #include "osl/move_action/store.h"
 #include "osl/move_classifier/check_.h"
+#include "osl/eval/see.h"
 #include "osl/rating/ratedMoveVector.h"
+#include "osl/record/csa.h"
 #include <boost/foreach.hpp>
 #include <iostream>
 
@@ -30,15 +37,11 @@ const size_t debug_time = NoDebug;
 #endif
 
 #define LEARN_PVS
+#define FOLLOW_BOOK
 
-std::ostream& 
-gpsshogi::operator<<(std::ostream& os, 
-		     const gpsshogi::PVVector& pv)
-{
-  BOOST_FOREACH(Move m, pv)
-    os << m << ' ';
-  return os << "\n";
-}
+const int MT_CAP_DRAGON = 955 + 647;
+const int EFUTIL_MG1 = MT_CAP_DRAGON*2/8;
+const int EFUTIL_MG2 = MT_CAP_DRAGON*2/8;
 
 gpsshogi::
 Table::Table()
@@ -62,13 +65,33 @@ Table::clear()
   table.clear();
 }
 
+
+/* ------------------------------------------------------------------------- */
+
+gpsshogi::
+BigramTable::BigramTable()
+{
+}
+
+gpsshogi::
+BigramTable::~BigramTable()
+{
+}
+
+void gpsshogi::
+BigramTable::clear()
+{
+  table.clear();
+}
+
 /* ------------------------------------------------------------------------- */
 
 gpsshogi::
 Quiesce::Quiesce(Eval *e, int a, int q) 
   : eval(e), all_moves_depth(a), quiesce_depth(q),
-    root_depth_left(0), time(0), pv(64), node_count(0)
+    root_depth_left(0), root_history_size(0), time(0), pv(64), node_count(0)
 {
+  setDepth(a, q);
 }
 
 gpsshogi::
@@ -81,6 +104,7 @@ Quiesce::clear()
 {
   node_count = 0;
   table.clear();
+  bigram_table.clear();
 }
     
 bool gpsshogi::
@@ -97,19 +121,26 @@ Quiesce::quiesce(NumEffectState& state,
   assert(alpha != beta);
   history = pv;
 
-  const Player turn = state.getTurn();
+  const Player turn = state.turn();
   if (eval::betterThan(turn, alpha, beta))
     std::swap(alpha, beta);
 
   key = HashKey(state);
   history_state.setRoot(state);
-  eval_value.reset(eval->newStack(state));
-
+  if (! eval_value)
+    eval_value.reset(eval->newStack(state));
+  else
+    eval_value->reset(state);
   root_depth_left = all_moves_depth + quiesce_depth;
+  if (! bonanzaCompatible())
+    assert(root_depth_left < PvMaxDepth/2);
+  else
+    assert(root_depth_left < PvMaxDepth);
   value = search(alpha, beta, root_depth_left);
   assert(history.size() < this->pv.size());
   pv = this->pv[history.size()];
-
+  root_history_size = history.size();
+  killers.fill(Move());
   return true;
 }
 
@@ -125,6 +156,19 @@ Quiesce::selectSeePlus(const NumEffectState& state, const MoveVector& src, MoveV
     moves.push_back(RatedMove(src[i], see));
   }
   moves.sort();
+  if (! history.empty() && history.back().isNormal()) {
+    const Square last_to = history.back().to();
+    for (size_t i=0; i<moves.size(); ++i) {
+      if (moves[i].move().to() == last_to) {
+	dst.push_back(moves[i].move());
+	std::rotate(moves.begin()+(int)i, moves.begin()+(int)i+1,
+		    moves.end());
+	assert(moves.back().move() == dst.back());
+	moves.pop_back();
+	break;
+      }
+    }
+  }
   for (size_t i=0; i<moves.size(); ++i)
     dst.push_back(moves[i].move());
 }
@@ -132,59 +176,125 @@ Quiesce::selectSeePlus(const NumEffectState& state, const MoveVector& src, MoveV
 void gpsshogi::
 Quiesce::generateTacticalMoves(MoveVector& out) const
 {
-  const Player turn = state()->getTurn();
+#ifdef FOLLOW_BOOK
+  if (! bonanzaCompatible()) {
+    static const BookInMemory& book = BookInMemory::instance();
+    book.find(key, out);
+    if (out.size() > 2)
+      out.resize(2);
+    if (! out.empty())
+      return;
+  }
+#endif
+  const Player turn = state()->turn();
   MoveVector moves;
   {
     move_action::Store store(moves);
     // capture
     for (int i=0; i<Piece::SIZE; ++i) {
-      const Piece piece = state()->getPieceOf(i);
+      const Piece piece = state()->pieceOf(i);
       if (! piece.isOnBoardByOwner(alt(turn)))
 	continue;
 
-      move_generator::GenerateCapture::generate(turn, *state(), piece.position(), 
+      move_generator::GenerateCapture::generate(turn, *state(), piece.square(), 
 						store);
     }
     // promote
-    move_generator::GeneratePromote<true>::generate(turn, *state(), store);    
+    const int quiesce_height = history.size() - root_history_size - all_moves_depth;
+    if (bonanzaCompatible() && quiesce_height > 6) {
+      if (state()->turn() == BLACK)
+	move_generator::Promote<BLACK>::generateMovesPtype<move_action::Store,PAWN>(*state(), store);
+      else 
+	move_generator::Promote<WHITE>::generateMovesPtype<move_action::Store,PAWN>(*state(), store);
+    }
+    else
+      move_generator::GeneratePromote<true>::generate(turn, *state(), store);
   }
   selectSeePlus(*state(), moves, out);
 }
 
 void gpsshogi::
-Quiesce::generateAllMoves(MoveVector& moves) const
+Quiesce::generateAllMoves(MoveVector& moves, int depth_left) const
 {
-  GenerateAllMoves::generate(state()->getTurn(), *state(), moves);
-  search::SortCaptureMoves::sortByTakeBack(*state(), moves);
-}
-void gpsshogi::
-Quiesce::generateAllMovesSeePlus(MoveVector& moves, int threshold) const
-{
-  MoveVector tmp;
-  GenerateAllMoves::generate(state()->getTurn(), *state(), tmp);
-  selectSeePlus(*state(), tmp, moves, threshold);
+  if (bonanzaCompatible()) {
+    MoveVector all;
+    LegalMoves::generate(*state(), all);
+    selectSeePlus(*state(), all, moves, -40000);
+    return;
+  }
+#ifdef FOLLOW_BOOK
+  {
+    static const BookInMemory& book = BookInMemory::instance();
+    book.find(key, moves);
+    if (moves.size() > 4)
+      moves.resize(4);
+    if (! moves.empty())
+      return;
+  }
+#endif
+  MoveLogProbVector all;
+#ifdef USE_OLD_PROBABILITY
+  static const rating::FeatureSet& feature_set
+    = rating::StandardFeatureSet::instance();
+  rating::RatingEnv env;
+  env.make(*state(), state()->pin(state()->turn()),
+	   state()->pin(alt(state()->turn())),
+	   eval_value->progress16());
+  feature_set.generateLogProb(*state(), env, 800, all, false);
+#else
+  static const move_probability::StandardFeatureSet& feature_set
+    = move_probability::StandardFeatureSet::instance();
+  MoveStack h;
+  if (! history.empty())
+    h.push(history[history.size()-1]);
+  Move threatmate = move_probability::StateInfo::findShortThreatmate(*state(), h.lastMove());
+  move_probability::StateInfo info(*state(), eval_value->progress16(),
+				   h, threatmate);
+  feature_set.generateLogProb2(info, all);
+#endif
+  const size_t width = (depth_left == root_depth_left) ? 8 : 4;
+  // const size_t width = (depth_left + all_moves_depth/2 >= root_depth_left) ? 8 : 4;
+  for (size_t i=0; i<std::min(all.size(), width); ++i)
+    moves.push_back(all[i].move());
 }
 
 void gpsshogi::
-Quiesce::generateTakeBack(MoveVector& out, Position to) const
+Quiesce::generateTakeBack(MoveVector& out, Square to) const
 {
   if (to.isPieceStand())
     return;
   MoveVector moves;
   {
     move_action::Store store(moves);
-    move_generator::GenerateCapture::generate(state()->getTurn(), *state(), to, store);
+    move_generator::GenerateCapture::generate(state()->turn(), *state(), to, store);
   }
 
   selectSeePlus(*state(), moves, out);
 }
 
+osl::Move gpsshogi::
+Quiesce::makeTakeBack(Move last_move) const
+{
+  if (! last_move.isNormal())
+    return Move();
+  const NumEffectState& state = *this->state();
+  const Player turn = state.turn();
+  Piece p = state.findCheapAttack(turn, last_move.to());
+  if (! p.isPiece() || state.pin(turn).test(p.number()))
+    return Move();
+  const bool promotable = last_move.to().canPromote(turn)
+    && canPromote(p.ptype());
+  return Move(p.square(), last_move.to(),
+	      promotable ? promote(p.ptype()) : p.ptype(),
+	      last_move.ptype(), promotable, turn);
+}
+
 void gpsshogi::
 Quiesce::generateEscapeFromLastMove(MoveVector& moves, Move last_move) const
 {
   typedef osl::PieceEval eval_t;
   MoveVector all;
-  if (state()->getTurn() == BLACK)
+  if (state()->turn() == BLACK)
     search::QuiescenceGenerator<BLACK>::escapeFromLastMove<eval_t>(*state(), last_move, all);
   else
     search::QuiescenceGenerator<WHITE>::escapeFromLastMove<eval_t>(*state(), last_move, all);
@@ -198,6 +308,11 @@ Quiesce::search(int alpha, int beta, int depth_left)
   ++node_count;
   const bool in_pv = alpha != beta;
   const size_t my_time = ++time;
+  const int full_search_depth_left
+    = std::max(0, depth_left - (root_depth_left - all_moves_depth));
+  const bool is_king_in_check = state()->inCheck();
+  const bool consider_standpat
+    = (bonanzaCompatible() || ! is_king_in_check) && ! full_search_depth_left;
 #ifdef DEBUG_QUIESCE
   if (my_time == debug_time) {
     Record *record = table.allocate(key);
@@ -211,23 +326,31 @@ Quiesce::search(int alpha, int beta, int depth_left)
     std::cerr << "\nnew node " << (in_pv ? "pv " : "") << depth_left << " t " 
 	      << my_time << " [" << alpha << " " << beta << "] " << history;
 #endif
-  const bool in_full_search = root_depth_left - all_moves_depth < depth_left;
-
-  const Player turn = state()->getTurn();
+  const Player turn = state()->turn();
   assert(! eval::betterThan(turn, alpha, beta));
   if (state()->inCheck(alt(turn)))
     return infty(turn);
+  if (history.size() > PvMaxDepth/8*7) {
+    std::cerr << history_state.initialState();
+    std::cerr << "warn long pv (" << alpha << ',' << beta << ")"
+	      << ' ' << eval_value->value() << ' '
+	      << (bonanzaCompatible() ? "B" : "")
+	      << (consider_standpat ? "S" : "");
+    BOOST_FOREACH(Move move, history)
+      std::cerr << ' ' << record::csa::show(move);
+    std::cerr << "\n";
+    return state()->inCheck() ? infty(alt(turn)) : infty(turn);
+  }
 
   const int initial_alpha = alpha;
   int best_value = -infty(turn);
-  const bool is_king_in_check = state()->inCheck();
   Move best_move;
   assert(history.size() < this->pv.size());
   PVVector& cur_pv = pv[history.size()];
   cur_pv = history;
   // TODO: tsumero?
   const int stand_pat = eval_value->value();
-  if (! is_king_in_check && ! in_full_search) {
+  if (consider_standpat) {
     // assert(eval_value->value() == eval->eval(*state()));
 #ifdef DEBUG_QUIESCE
     if (my_time == debug_time)
@@ -243,6 +366,21 @@ Quiesce::search(int alpha, int beta, int depth_left)
       alpha = eval::max(turn, alpha, best_value);
     }
   }
+  // delayed futility pruning 
+  const Move last_move = history.empty() ? Move() : history.back();
+  if (bonanzaCompatible()
+      && ! state()->inCheck() && ! state()->wasCheckEvasion(last_move)) {
+    if (full_search_depth_left == 1) {
+      const int value = stand_pat - EFUTIL_MG1 * eval::delta(turn);
+      if (eval::betterThan(turn, value, beta))
+	return value;
+    }
+    else if (full_search_depth_left == 2) {
+      const int value = stand_pat - EFUTIL_MG2 * eval::delta(turn);
+      if (eval::betterThan(turn, value, beta))
+	return value;
+    }    
+  }
 
   // table
   Record *record = table.allocate(key);
@@ -300,41 +438,65 @@ Quiesce::search(int alpha, int beta, int depth_left)
   }
 
   // search specials
-  FixedCapacityVector<Move,2> first_moves;
+  FixedCapacityVector<Move,4> first_moves;
   if (record->best_move.isNormal())
-    if (in_full_search 
-	|| (depth_left > 0 && record->best_move.capturePtype() != PTYPE_EMPTY))
+    if (full_search_depth_left 
+	|| (! bonanzaCompatible() && depth_left > 0
+	    && (history.size() - root_history_size <= root_depth_left+2)
+	    && (eval::See::see(*state(), record->best_move) >= 0)))
       first_moves.push_back(record->best_move);
-#ifdef LEARN_DEEP_SEARCH
-  if (! state()->inCheck() && in_full_search && eval::betterThan(turn, stand_pat, alpha))
-    first_moves.push_back(Move::PASS(turn));
-#endif
+  if (full_search_depth_left && !history.empty()) {
+    Move bigram_killer = bigram_table.find(*state(), history.back());
+    if (bigram_killer.isNormal() && !first_moves.isMember(bigram_killer))
+      first_moves.push_back(bigram_killer);
+  }
+  Move takeback = makeTakeBack(last_move);
+  if (takeback.isNormal()
+      && eval::See::see(*state(), takeback) >= 0
+      && ! first_moves.isMember(takeback)) {
+    assert(state()->isAlmostValidMove<false>(takeback));
+    first_moves.push_back(takeback);
+  }
+  Move killer_move = killers[history.size()-root_history_size];
+  if (killer_move.isNormal()
+      && state()->isAlmostValidMove<false>(killer_move)
+      && (full_search_depth_left 
+	  || (! bonanzaCompatible() && depth_left > 0 
+	      && eval::See::see(*state(), killer_move) > 0)))
+    if (! first_moves.isMember(killer_move))
+      first_moves.push_back(killer_move);
   for (size_t i=0; i<first_moves.size(); ++i) {
     const Move m = first_moves[i];
     const int reduction = m.isPass() ? 2 : 1;
     assert(eval::notLessThan(turn, alpha, best_value));
     assert(eval::notLessThan(turn, beta, alpha));
+    const bool extension = full_search_depth_left > 0
+      && ! history.empty()
+      && history.size() < PvMaxDepth/4
+      && history[history.size()-1].isNormal()
+      && m.to() == history[history.size()-1].to()
+      && eval::See::see(*state(), m) > 0;
 
     int value;
     {
       const HashKey old_hash = key;
       DoUndoMoveLock lock(history_state, m);
       history.push_back(m);
-
+      
       key = old_hash.newHashWithMove(m);
       eval_value->push(*state(), m);
 #ifdef LEARN_PVS
       if (in_pv && i > 0 && root_depth_left == depth_left) 
       {
-	value = search(alpha+eval::delta(turn), alpha, depth_left-reduction);
+	value = search(alpha+eval::delta(turn), alpha, depth_left-reduction+extension);
 	if (eval::betterThan(turn, value, alpha)) {
-	  value = search(beta, alpha, depth_left-reduction);
+	  value = search(beta, alpha, depth_left-reduction+extension);
 	}
       }
       else 
 #endif
       {
-	value = search(beta, alpha, depth_left-reduction);
+	value = search(beta, alpha, depth_left-reduction+extension);
       }
       key = old_hash;
       eval_value->pop();
@@ -344,6 +506,7 @@ Quiesce::search(int alpha, int beta, int depth_left)
       assert(history.size()+1 < this->pv.size());
       cur_pv = pv[history.size()+1];
       if (eval::betterThan(turn, value, alpha)) {
+	killers[history.size()-root_history_size] = m;
 	if (eval::notLessThan(turn, value, beta)) {
 	  record->setLowerBound(my_time, depth_left, value, in_pv);
 	  record->best_move = m;
@@ -358,8 +521,20 @@ Quiesce::search(int alpha, int beta, int depth_left)
 
   // move generation
   MoveVector moves;
-  if (is_king_in_check) {
-    GenerateEscapeKing::generateCheap(*state(), moves);
+  if (is_king_in_check && ! bonanzaCompatible()) {
+    const bool need_selection = history.size() >= 2
+      && history[history.size()-1].isNormal()
+      && history[history.size()-2].isNormal()
+      && history[history.size()-1].to() == history[history.size()-2].to()
+      && history[history.size()-2].isDrop();
+    MoveVector all;
+    GenerateEscapeKing::generateCheap(*state(), all);
+    BOOST_FOREACH(Move move, all) {
+      if (need_selection && move.isDrop()
+	  && ! state()->hasEffectAt(state()->turn(), move.to()))
+	continue;		// successive sacrifice
+      moves.push_back(move);
+    }
     if (moves.empty()) {
       record->setValue(my_time, checkmate_special_depth, -infty(turn));
       return -infty(turn);
@@ -367,19 +542,16 @@ Quiesce::search(int alpha, int beta, int depth_left)
     move_order::CaptureSort::sort(moves.begin(), moves.end());
   }
   else {
-    if (in_full_search) {
-#ifdef LEARN_DEEP_SEARCH
-      generateAllMovesSeePlus(moves, -200*(all_moves_depth - (root_depth_left - depth_left)));
-#else
-      generateAllMoves(moves);
-#endif
+    if (full_search_depth_left) {
+      generateAllMoves(moves, depth_left);
     }
     else if (depth_left > 0) {
       generateTacticalMoves(moves);
-      if (root_depth_left == all_moves_depth + depth_left
+      if (! bonanzaCompatible()
+	  && root_depth_left == all_moves_depth + depth_left
 	  && ! history.empty())
-	generateEscapeFromLastMove(moves, history.back());
-    } else {
+	generateEscapeFromLastMove(moves, last_move);
+    } else if (! bonanzaCompatible()) {
       generateTakeBack(moves, history[history.size()-1].to());
     }
   }
@@ -388,14 +560,18 @@ Quiesce::search(int alpha, int beta, int depth_left)
     std::cerr << "time " << my_time << " best_move " << best_move << " " << record->best_move << "\n";
 #endif  
   // search all
-  if (first_moves.empty())
-    first_moves.push_back(Move::PASS(turn));
   const int reduction = (is_king_in_check && moves.size() == 1) ? 0 : 1;
   for (MoveVector::const_iterator p=moves.begin(); p!=moves.end(); ++p) {
-    if (first_moves[0] == *p)
+    if (first_moves.isMember(*p))
       continue;
     if (osl::search::ShouldPromoteCut::canIgnoreAndNotDrop(*p))
       continue;
+    const bool extension = full_search_depth_left > 0
+      && reduction > 0 && ! history.empty()
+      && history.size() < PvMaxDepth/4
+      && history[history.size()-1].isNormal()
+      && p->to() == history[history.size()-1].to()
+      && See::see(*state(), *p) > 0;
 #ifdef DEBUG_QUIESCE
     if (my_time == debug_time)
       std::cerr << "try " << *p << " " << cur_pv;
@@ -414,15 +590,15 @@ Quiesce::search(int alpha, int beta, int depth_left)
 #ifdef LEARN_PVS
       if (in_pv && root_depth_left == depth_left) 
       {
-	value = search(alpha+eval::delta(turn), alpha, depth_left-reduction);
+	value = search(alpha+eval::delta(turn), alpha, depth_left-reduction+extension);
 	if (eval::betterThan(turn, value, alpha)) {
-	  value = search(beta, alpha, depth_left-reduction);
+	  value = search(beta, alpha, depth_left-reduction+extension);
 	}
       }
       else 
 #endif
       {
-	value = search(beta, alpha, depth_left-reduction);
+	value = search(beta, alpha, depth_left-reduction+extension);
       }
 #ifdef DEBUG_QUIESCE
       if (debug_time != NoDebug && time >= debug_time+1)
@@ -437,6 +613,11 @@ Quiesce::search(int alpha, int beta, int depth_left)
       assert(history.size()+1 < this->pv.size());
       cur_pv = pv[history.size()+1];
       if (eval::betterThan(turn, value, alpha)) {
+	killers[history.size()-root_history_size] = *p;
+#if 1
+	if (! history.empty())
+	  bigram_table.add(history.back(), *p);
+#endif
 	if (eval::notLessThan(turn, value, beta)) {
 	  record->setLowerBound(my_time, depth_left, value, in_pv);
 	  record->best_move = *p;
