@@ -29,6 +29,7 @@
 #include "osl/record/opening/openingBook.h"
 #include "osl/game_playing/alphaBetaPlayer.h"
 #include "osl/game_playing/gameState.h"
+#include "osl/misc/filePath.h"
 #include "osl/misc/random.h"
 #include "osl/misc/sjis2euc.h"
 #include "osl/search/moveGenerator.h"
@@ -319,7 +320,7 @@ MySession::open(const std::vector<std::string>& params)
 
   if (boost::algorithm::iequals(".csa", ext))
   {
-    const osl::CsaFile file(path.native_file_string());
+    const osl::CsaFile file(osl::misc::file_string(path));
     const osl::record::Record record = file.getRecord();
     osl::vector<osl::Move> moves;
     osl::vector<int> time;
@@ -335,20 +336,20 @@ MySession::open(const std::vector<std::string>& params)
   }
   else if (boost::algorithm::iequals(".usi", ext))
   {
-    const osl::UsiFile file(path.native_file_string());
+    const osl::UsiFile file(osl::misc::file_string(path));
     const osl::vector<osl::Move> moves = file.getRecord().getMoves();
     board->setMoves(file.getInitialState(), moves);
   }
   else if (boost::algorithm::iequals(".ki2", ext))
   {
-    const osl::Ki2File file(path.native_file_string());
+    const osl::Ki2File file(osl::misc::file_string(path));
     const osl::vector<osl::Move> moves = file.getRecord().getMoves();
     board->setMoves(file.getInitialState(), moves);
   }
   else if (boost::algorithm::iequals(".kif", ext) 
-	   && osl::KakinokiFile::isKakinokiFile(path.native_file_string()))
+	   && osl::KakinokiFile::isKakinokiFile(osl::misc::file_string(path)))
   {
-    const osl::KakinokiFile file(path.native_file_string());
+    const osl::KakinokiFile file(osl::misc::file_string(path));
     const osl::record::Record record = file.getRecord();
     osl::vector<osl::Move> moves;
     osl::vector<int> time;
@@ -363,7 +364,7 @@ MySession::open(const std::vector<std::string>& params)
     size_t index = 0;
     if (params.size() == 3)
       index = boost::lexical_cast<size_t>(params[2]);
-    osl::KisenFile file(path.native_file_string());
+    osl::KisenFile file(osl::misc::file_string(path));
     osl::KisenIpxFile ipx(file.ipxFileName());
     if (index >= file.size())
     {
@@ -439,11 +440,11 @@ MySession::openurl(const std::vector<std::string>& params)
   const std::string& url = params[1];
   const boost::filesystem::path temp_file 
     = cache_dir / boost::algorithm::replace_all_copy(url, "/", "_");
-  if (gpsshell::getFileOverHttp(url, temp_file.native_file_string()))
+  if (gpsshell::getFileOverHttp(url, osl::misc::file_string(temp_file)))
     return;
   std::vector<std::string> tokens_to_pass;
   tokens_to_pass.push_back("open");
-  tokens_to_pass.push_back(temp_file.native_file_string());
+  tokens_to_pass.push_back(osl::misc::file_string(temp_file));
   if (params.size() == 3)
     tokens_to_pass.push_back(params[2]); // index
   open(tokens_to_pass);
@@ -1235,14 +1236,14 @@ MySession::openingInclude(const std::vector<std::string>& params)
 
   try
   {
-    const osl::CsaFile file(path.native_file_string());
+    const osl::CsaFile file(osl::misc::file_string(path));
     const osl::record::Record record = file.getRecord();
     const osl::vector<osl::Move> moves = record.getMoves();
 
     if (nth_move > moves.size())
     {
       std::cerr << boost::format("Invalid argument. %d is too large for the file %s [%d moves]\n")
-                   % nth_move % path.native_file_string() % moves.size();
+                   % nth_move % osl::misc::file_string(path) % moves.size();
       std::cerr << USAGE;
       return;
     }
@@ -1268,7 +1269,7 @@ MySession::openingInclude(const std::vector<std::string>& params)
   }
   catch (osl::record::csa::CsaIOError& e)
   {
-    std::cerr << boost::format("Failed to open the csa file: %s\n") % path.native_file_string();
+    std::cerr << boost::format("Failed to open the csa file: %s\n") % osl::misc::file_string(path);
     std::cerr << USAGE;
     return;
   }
@@ -1356,7 +1357,7 @@ MySession::ignoreListShow(const std::vector<std::string>& params)
   const gpsshell::IgnoreList::index_t index = the_ignorelist.getCurrentIndex();
   std::vector<std::string> args;
   args.push_back("open");
-  args.push_back(index.get<0>().file_string());
+  args.push_back(osl::misc::file_string(index.get<0>()));
   args.push_back(boost::lexical_cast<std::string>(index.get<1>()));
   const std::string& comment = index.get<2>();
   if (!comment.empty())
