@@ -122,6 +122,11 @@ MainWindow::MainWindow(QWidget *parent, const char *name)
   connect(saveToAction, SIGNAL(activated()), viewer, SLOT(saveMovesToCurrent()));
   saveToAction->addTo(fileMenu);
 
+  QAction *autoSaveMovesAction = new QAction(tr("&Auto Save Moves..."), this);
+  connect(autoSaveMovesAction, SIGNAL(activated()), viewer, SLOT(autoSaveMoves()));
+  autoSaveMovesAction->addTo(fileMenu);
+  autoSaveMovesAction->setWhatsThis(tr("Select a file where moves played in the Kifu viewer are saved automatically."));
+
   QAction *exportAction = new QAction(tr("&Export"), this);
   connect(exportAction, SIGNAL(activated()), viewer, SLOT(exportCurrent()));
   exportAction->addTo(fileMenu);
@@ -268,7 +273,7 @@ MainWindow::MainWindow(QWidget *parent, const char *name)
 	  SLOT(enableEffect(bool)));
   effectAction->setToggleAction(true);
   effectAction->addTo(viewMenu);
-  QAction *evalGraphAction = new QAction(tr("Evaluation Graph"), this);
+  QAction *evalGraphAction = new QAction(tr("Evaluation &Graph"), this);
   evalGraphAction->addTo(viewMenu);
   connect(evalGraphAction, SIGNAL(activated()), viewer, SLOT(openEvalGraph()));  
   lastMoveAction = new QAction(tr("Highlight &Last Move"), this);
@@ -285,6 +290,13 @@ MainWindow::MainWindow(QWidget *parent, const char *name)
   bookMoveAction->addTo(viewMenu);
   bookMoveAction->toggle();
 
+  arrowMoveAction = new QAction(tr("Show &Arrows for Moves"), this);
+  connect(arrowMoveAction, SIGNAL(toggled(bool)), viewer,
+	  SLOT(showArrowMove(bool)));
+  arrowMoveAction->setToggleAction(true);
+  arrowMoveAction->addTo(viewMenu);
+  arrowMoveAction->toggle();
+
   menuBar()->insertItem(tr("&File"), fileMenu);
   menuBar()->insertItem(tr("&Edit"), editMenu);
   menuBar()->insertItem(tr("&Go"), goMenu);
