@@ -24,7 +24,7 @@ public:
   void showRecord(const osl::hash::HashKey& key, const osl::SimpleState&,
 		  const osl::search::SimpleHashTable *table);
 protected:
-  void showContextMenu(Q3PopupMenu *);
+  void contextMenuEvent(QContextMenuEvent *);
 private slots:
   void showRecord();
 private:
